/**
  **********************************************************************************
  * @file     IQS266.cpp
  * @author   Azoteq
  * @version  V0.0.0
  * @date     2017-08-23
  * @brief    This file contains the constructors and methods which allow ease of
  *           use of an IQS266 capactive touch controller. The IQS266 is a capacitive
  *           touch Integrated Circuit (IC) which provides easy 2x3 trackpad
  *           functionality. This class provides an easy means of initializing
  *           and interacting with the IQS266 device from an Arduino.
  **********************************************************************************
  * @attention  Makes use of the following standard Arduino libraries:
  * - Arduino.h -> Included in IQS266.h, comes standard with Arduino
  * - Wire.h    -> Included in IQS266.h, comes standard with Arduino
  *
  **********************************************************************************
  */

// Include Files
#include "IQS266.h"

// Private Definitions

// Private Global Variables


/**************************************************************************************************************/
/*                                                CONSTRUCTORS                                                */
/**************************************************************************************************************/
IQS266::IQS266(){
}

/**************************************************************************************************************/
/*                                              PUBLIC METHODS                                                */
/**************************************************************************************************************/

/**
  * @name   begin
  * @brief  A method to initialize the IQS266 device with the device address and ready pin specified by the user.
  * @param  deviceAddress -> The address of the IQS266 device.
  *         readyPin      -> The Arduino pin which is connected to the ready pin of the IQS266 device.
  * @retval Returns true if communication has been successfully established, returns false if not.
  * @notes  Receiving a true return value does not mean that initialization was successful.
  *         Receiving a true return value only means that the IQS device responded to the request for communication.
  *         Receiving a false return value means that initialization did not take place at all.
  *         If communication is successfully established then it is unlikely than initialization will fail.
  */
bool IQS266::begin(uint8_t deviceAddressIn, uint8_t readyPinIn)
{
  // Initialize I2C communication here, since this library can't function without it.
  Wire.begin();
  Wire.setClock(400000);

  bool response = false;  // The return value. Is set to the return value of the requestComms function.
  _deviceAddress = deviceAddressIn;
  _readyPin = readyPinIn;

  // Request communication and run ATI routine.
  response = requestComms();

  if(response)
  {
	acknowledgeReset(RESTART);
	setReportRateLP(8, RESTART);
	setNPSegmentRate(6, RESTART);
    autoTune(STOP);
  }

  return response;
}

/**
  * @name   requestComms
  * @brief  A method to request communication by briefly pulling the READY pin of the IQS266 LOW and waiting for the IQS266 device to respond, 
  *         the response by the IQS266 is done by pulling the READY pin LOW again once the MCU has released it.
  *         If the iqs266 device does not respond in 10ms the ready is pulled low again, this repeats 10 times before a false value is returned.
  *         Thus, if no response is received within 100ms then the method returns false. If a response is received then the method returns true.
  * @param  None.
  * @retval Boolean: True if a response is received within 100ms, false if not.
  * @notes  Use this function when the master must initiate communication. 
  *         It is useful when the device is in event mode and new settings need to loaded onto the device.
  */
bool IQS266::requestComms(void)
{
  bool response = false;        // The return value. Is set to true if a response is received within 100ms.
  uint16_t notReadyCount = 0 ;  // Increments every time the loop executes to keep track of how long the request is going on.
  
  // Pull the ready bus LOW to let the IQS266 device know you want to communicate.
  toggleReady();
  
  // Wait for the IQS266 to respond by pulling the ready bus LOW. Redo request every 10ms, return false after 100ms.
  while(digitalRead(_readyPin))
  {
    notReadyCount++;
    delayMicroseconds(100);
    
    if((notReadyCount%1000) == 0)
      return response;
    if((notReadyCount%100) == 0)
      toggleReady();
  }
  // If the processing reaches this point then a response has been received.
  response = true;
  return response;
}

/**
  * @name  waitForReady
  * @brief  A method which waits for the IQS266 device to pull the ready pin low.
  *     If there is no response for 100ms a false value is returned and the user can then either force communication 
  *     by using the requestComms function or just wait longer by placing this method in a while loop.
  * @param  None.
  * @retval Boolean: True if a response is received within 100ms, false if not.
  * @notes  Use this function when the device is in stream mode and you want the master to await communication from the IQS266 device.
  *     If you want the master to initiate communication use the requestComms function.
  */
bool IQS266::waitForReady(void)
{
  bool readyLow = false;      // The return value. Set to true if the ready pin is pulled low by the IQS266 within 100ms.
  uint16_t notReadyCount = 0 ;  // Increments every time the loop executes to keep track of how long the request is going on.
  
  // Wait for communication from IQS266 device. Timeout after 100ms.
  while(digitalRead(_readyPin))
  {
    notReadyCount++;
    delayMicroseconds(100);
    
    if((notReadyCount%1000) == 0)
      return readyLow;
  }
  // If the processing gets here then a response has been received.
  readyLow = true;
  return readyLow;
}

/**
  * @name	checkReset
  * @brief  A method which checks if the device has reset and returns the reset status.
  * @param  None.
  * @retval Returns true if a reset has occurred, false if no reset has occurred.
  * @notes  If a reset has occurred the device settings should be reloaded using the begin function.
  * 		After new device settings have been reloaded the acknowledge reset function can be used
  * 		to clear the reset flag.
  */
bool IQS266::checkReset(bool stopOrRestart)
{
	uint8_t transferBytes[1];	// A temporary array to hold the byte to be transferred.
								// Use an array to be consistent with other methods of the library.
	// Read the System Flags from the IQS266.
	readRandomBytes(SYS_FLAGS, 1, transferBytes, stopOrRestart);
	transferBytes[0] &= SHOW_RESET_BIT;
	// Return the reset status.
	if(transferBytes[0] != 0)
		return true;
	else
		return false;
}

/**
  * @name	acknowledgeReset
  * @brief  A method which clears the Show Reset bit by writing it to a 0.
  * @param  None.
  * @retval None.
  * @notes  If a reset has occurred the device settings should be reloaded using the begin function.
  * 		After new device settings have been reloaded this method should be used to clear the
  * 		reset bit.
  */
void IQS266::acknowledgeReset(bool stopOrRestart)
{
	uint8_t transferBytes[2];	// A temporary array to hold the bytes to be transferred.
	// Read the System Flags from the IQS266, these must be read first in order not to change any settings.
	// We are interested in the 2nd byte at the address location, therefore, we must read and write both bytes.
	readRandomBytes(PROXSETTINGS_23, 2, transferBytes, RESTART);
	// SWrite the AAck Reset bit to 1 to clear the Show Reset Flag.
	transferBytes[1] |= ACK_RESET_BIT;
	// Write the new byte to the System Flags address.
	writeRandomBytes(PROXSETTINGS_23, 2, transferBytes, stopOrRestart);
}

/**
  * @name   autoTune
  * @brief  A method which sets the REDO_ATI_BIT in order to force the IQS266 device to run the 
  *         Automatic Tuning Implementation (ATI) routine.
  * @param  None.
  * @retval None.
  * @notes  To force ATI, bit 4 in PROX_SETTINGS_0 is set. PROX_SETTINGS_0 is byte 0 in the PROXSETTINGS_0_1 address.
  */
void IQS266::autoTune(bool stopOrRestart)
{
  uint8_t transferByte[1]; // Array to store the bytes transferred.
  	  	  	  	  	  	   // Use an array to be consistent with other methods in this class.
  
  // Read the current PROX_SETTINGS_0 value in order not to alter any existing settings.
  readRandomBytes(PROXSETTINGS_01, 1, transferByte, RESTART);
  // Mask the settings with the REDO_ATI_BIT.
  transferByte[0] |= REDO_ATI_BIT;  // This is the bit required to start an ATI routine.
  // Write the new byte to the required device.
  writeRandomBytes(PROXSETTINGS_01, 1, transferByte, stopOrRestart);
}

/**
  * @name   setStreamMode
  * @brief  A method which sets the IQS266 device into stream mode.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *                          Use the STOP and RESTART definitions.
  * @retval None. 
  * @notes  All other bits at the register address are preserved.
  */
void IQS266::setStreamMode(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The array which will hold the bytes which are transferred.

  // First read the bytes at the memory address so that they can be preserved.
  readRandomBytes(PROXSETTINGS_01, 2, transferBytes, RESTART);
  // Clear the EVENT_MODE_BIT in PROX_SETTINGS_1
  transferBytes[1] &= ~(EVENT_MODE_BIT);
  // Write the bytes back to the device
  writeRandomBytes(PROXSETTINGS_01, 2, transferBytes, stopOrRestart);
}

/**
  * @name   setEventMode
  * @brief  A method to set the IQS266 device into event mode.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *              Use the STOP and RESTART definitions.
  * @retval None. 
  * @notes  All other bits at the register address are preserved.
  */
void IQS266::setEventMode(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The array which will hold the bytes which are transferred.

  // First read the bytes at the memory address so that they can be preserved.
  readRandomBytes(PROXSETTINGS_01, 2, transferBytes, RESTART);
  // Clear the EVENT_MODE_BIT in PROX_SETTINGS_1
  transferBytes[1] |= EVENT_MODE_BIT;
  // Write the bytes back to the device
  writeRandomBytes(PROXSETTINGS_01, 2, transferBytes, stopOrRestart);
}

/**
  * @name   readSystemFlags
  * @brief  A method which reads and returns the IQS266 system flags.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *              			 Use the STOP and RESTART definitions.
  * @retval Returns an unsigned char containing the system flags byte.
  * @notes  This does not alter the systemFlags union. To update the systemFlags union assign the return value to systemFlags.flagByte
  * 		or call the updateSystemFlags.
  */
uint8_t IQS266::readSystemFlags(bool stopOrRestart)
{
	uint8_t transferBytes[1];	// The array which will hold the bytes to be transferred.

	// Read the system flags.
	readRandomBytes(SYS_FLAGS, 1, transferBytes, stopOrRestart);
	// Return the system flags.
	return transferBytes[0];
}

/**
  * @name   updateSystemFlags
  * @brief  A method which reads the IQS266 system flags and assigns them to the systemFlags union.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *              			 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The systemFlags union is altered with the new value of the system flags register.
  */
void IQS266::updateSystemFlags(bool stopOrRestart)
{
	uint8_t transferBytes[1];	// The array which will hold the bytes to be transferred.

	// Read the system flags.
	readRandomBytes(SYS_FLAGS, 1, transferBytes, stopOrRestart);
	// Assign the system flags to the system flags union.
	systemFlags.flagByte =  transferBytes[0];
}

/**
  * @name   readEvents
  * @brief  A methods which reads and returns the byte which holds the event flags.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the event flags.
  * @notes  The user must determine which events occurred by use of the event but definitions.
  */
uint8_t IQS266::readEvents(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The temporary address which will hold the neccessary bytes from the SYS_FLAGS
  
  // Read the bytes using the readRandomByte method
  readRandomBytes(SYS_FLAGS, 2, transferBytes, stopOrRestart);
  // 2 bytes are now loaded into the transferBytes array, we are interested in the 2nd byte, so just return it.
  return transferBytes[1];
}

/**
  * @name   updateEvents
  * @brief  A method which reads the IQS266 event flags and assigns them to the eventFlags union.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or must be closed after this action.
  *              			 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The eventFlags union is altered with the new value of the event flags register..
  */
void IQS266::updateEvents(bool stopOrRestart)
{
	uint8_t transferBytes[2];	// The array which will hold the bytes to be transferred.

	// Read the event flags, byte 1 at SYS_Flags
	readRandomBytes(SYS_FLAGS, 2, transferBytes, stopOrRestart);
	// Assign the event flags to the system flags union.
	events.flagByte =  transferBytes[1];
}

/**
  * @name   readGestures
  * @brief  A method which reads and returns the byte which holds the gesture flags.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the gesture flags.
  * @notes  The user must determine which gestures occurred by use of the gesture bit definitions.
  * 		This method does not alter the gestures union. The returend value can be assigned to gestures.flagByte if the union must also be
  * 		updated, or the updateGestures method can be called.
  * 		This method is exactly the same as readTrackpadFlags and both methods return a value from the same register. It is up to the user
  * 		to decide which one to use.
  */
uint8_t IQS266::readGestures(bool stopOrRestart)
{
  uint8_t transferBytes[1]; // The temporary address which will hold the TP_FLAGS byte.
  
  // Read the byte using the readRandomByte method, only one byte is stored at the TOUCHPAD_FLAGS address.
  readRandomBytes(TP_FLAGS, 1, transferBytes, stopOrRestart);
  //  return the byte.
  return transferBytes[0];
}

/**
  * @name   updateGestures
  * @brief  A method which reads the gestures and assigns it to the gestures union.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The gestures union is altered with the new value at the trackpad flags register.
  * 		This method is exactly the same as updateTrackpadFlags, except it assigns to the gestures union.
  * 		This is done to allow a user to use either gestures or trackpad flags, but they are the same thing.
  * 		Updating the gestures union does not update the trackpadFlags union.
  */
void IQS266::updateGestures(bool stopOrRestart)
{
  uint8_t transferBytes[1]; // The temporary address which will hold the TP_FLAGS byte.

  // Read the byte using the readRandomByte method, only one byte is stored at the TP_FLAGS address.
  readRandomBytes(TP_FLAGS, 1, transferBytes, stopOrRestart);
  //  Assign the byte to the union.
  gestures.flagByte =  transferBytes[0];
}

/**
  * @name   readTrackpadFlags
  * @brief  A method which reads and returns the byte which holds the trackpad flags.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the trackpad flags.
  * @notes  This method does not alter the gestures union. The returned value can be assigned to trackpadFlags.flagByte if the union must also be
  * 		updated, or the updateTrackpadFlags method can be called.
  * 		This method is exactly the same as readGestures and both methods return a value from the same register. It is up to the user
  * 		to decide which one to use.
  */
uint8_t IQS266::readTrackpadFlags(bool stopOrRestart)
{
  uint8_t transferBytes[1]; // The temporary address which will hold the TP_FLAGS byte.

  // Read the byte using the readRandomByte method, only one byte is stored at the TOUCHPAD_FLAGS address.
  readRandomBytes(TP_FLAGS, 1, transferBytes, stopOrRestart);
  //  return the byte.
  return transferBytes[0];
}

/**
  * @name   updateTrackpadFlags
  * @brief  A method which reads the trackpad flags and assigns it to the trackpadFlags union.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The trackpadFlags union is altered with the new value at the trackpad flags register.
  * 		This method is exactly the same as updateGestures, except it assigns to the trackpadFlags union.
  * 		This is done to allow a user to use either gestures or trackpad flags, but they are the same thing.
  * 		Updating the trackpadFlags union does not update the gestures union.
  */
void IQS266::updateTrackpadFlags(bool stopOrRestart)
{
  uint8_t transferBytes[1]; // The temporary address which will hold the TP_FLAGS byte.

  // Read the byte using the readRandomByte method, only one byte is stored at the TP_FLAGS address.
  readRandomBytes(TP_FLAGS, 1, transferBytes, stopOrRestart);
  //  Assign the byte to the union.
  trackpadFlags.flagByte =  transferBytes[0];
}

/**
  * @name   readProxByte
  * @brief  A method which reads and returns the PROX CHANNEL byte which holds the prox flag for channel 0, this is a bit uselss as it only contains
			one bit which is set when the prox threshold on channel 0 is breached. It would probably be better to use the Prox Event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the prox flag for channel 0.
  * @notes  The CH0_PROX_BIT definition can be used to determine if there is a prox on channel 0.
  */
uint8_t IQS266::readProxByte(bool stopOrRestart)
{
  uint8_t transferBytes[1]; // The temporary address which will hold the PROX_CHANNELS byte.
  
  // Read the byte using the readRandomBytes method to read the first byte at the CHANNEL_BYTES address.
  readRandomBytes(CHANNEL_BYTES, 1, transferBytes, stopOrRestart);
  //  return the byte.
  return transferBytes[0];
}

/**
  * @name   readTouchByte
  * @brief  A method which reads and returns the TOUCH CHANNELS byte which holds the touch flag for all the channels.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the touch flags for all the channels.
  * @notes  The CHx_TOUCH_BIT definitions can be used to determine on which channels a touch is present.
  */
uint8_t IQS266::readTouchByte(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The temporary address which will hold the bytes from the CHANNEL_BYTES address.
  
  // Read the bytes using the readRandomBytes method to read both bytes at the CHANNEL_BYTES address.
  readRandomBytes(CHANNEL_BYTES, 2, transferBytes, stopOrRestart);
  // Return the second byte.
  return transferBytes[1];
}

/**
  * @name   readXCoordinate
  * @brief  A methodd which reads and returns the byte which holds the X-Coordinates of the touch taking place on the trackpad.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the X-Cordinates flags.
  * @notes  The user must determine which gestures occurred by use of the gesture bit definitions.
  */
uint8_t IQS266::readXCoordinate(bool stopOrRestart)
{
  uint8_t transferBytes[1]; // The temporary address which will hold the X_CURR byte.
  
  // Read the byte using the readRandomByte method.
  readRandomBytes(COORDINATES, 1, transferBytes, stopOrRestart);
  //  Return the byte, the X_CURR byte is the first byte at the COORDINATES address.
  return transferBytes[0];
}

/**
  * @name   readChannelStatus
  * @brief  A method which reads both the touch byte and the prox byte and returns the combined value.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The touch byte and prox byte are combined by setting the value of bit 7 of the touch byte to the value of bit 0 of the prox byte.
  * 		This is because the touch byte does not use bit 7 and the prox byte only usess bit 0. It is also conveniet because the touch and
  * 		and prox bytes use adjacent addresses.
  */
uint8_t IQS266::readChannelStatus(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The temporary address which will hold the Touch and Prox bytes.

  // Read the byte using the readRandomByte method, only one byte is stored at the TP_FLAGS address.
  readRandomBytes(CHANNEL_BYTES, 2, transferBytes, stopOrRestart);
  // Combine the 2 bytes, shift the proxbyte left 7 places and or it with the touch byte.
  transferBytes[1] |= (transferBytes[0]<<7);
  // Return the combined byte.
  return transferBytes[1];
}

/**
  * @name   updateChannelStatus
  * @brief  A method which read both the touch byte and the prox byte and assigns the channel status to the channels union.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The touch byte and prox byte are combined by setting the value of bit 7 of the touch byte to the value of bit 0 of the prox byte.
  * 		This is because the touch byte does not use bit 7 and the prox byte only usess bit 0. It is also conveniet because the touch and
  * 		and prox bytes use adjacent addresses.
  */
void IQS266::updateChannelStatus(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The temporary address which will hold the Touch and Prox bytes.

  // Read the byte using the readRandomByte method, only one byte is stored at the TP_FLAGS address.
  readRandomBytes(CHANNEL_BYTES, 2, transferBytes, stopOrRestart);
  // Combine the 2 bytes, shift the proxbyte left 7 places and or it with the touch byte.
  transferBytes[1] |= (transferBytes[0]<<7);
  //  Assign the combined byte to the union.
  channels.flagByte = transferBytes[1];
}

/**
  * @name   readYCoordinate
  * @brief  A methodd which reads and returns the byte which holds the Y-Coordinates of the touch taking place on the trackpad.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Unsigned Char: Returns the byte which contains the Y-Cordinates flags.
  * @notes  The user must determine which gestures occurred by use of the gesture bit definitions.
  */
uint8_t IQS266::readYCoordinate(bool stopOrRestart)
{
  uint8_t transferBytes[2]; // The temporary address which will hold the bytes at the COORDINATES address.
  
  // Read the bytes using the readRandomByte method.
  readRandomBytes(COORDINATES, 2, transferBytes, stopOrRestart);
  // Return the byte, the Y_CURR byte is the second byte at the COORDINATES address.
  return transferBytes[1];
}

/**
  * @name  setTouchSensitivity
  * @brief  A method which sets the touch threshold on all 7 touch channels to the same value, thereby adjusting the sensitivity of the channels.
  *         The Prox threshold for channel 0 is not altered at all.
  * @param  sensitivity   -> A value from 0 to 255 which specifies the touch threshold for the channels, a smaller value means more sensitive.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the writeRandomBytes methods.
  */
void IQS266::setTouchSensitivity(uint8_t sensitivity, bool stopOrRestart)
{
  uint8_t transferBytes[8]; // Temporary array which will hold the bytes to be transferred.
  
  // Read the channel 0 prox threshold from the THRESHOLDS_CH0 register
  readRandomBytes(THRESHOLDS_CH0, 1, transferBytes, RESTART);    // transferBytes[0] now holds the PROX_THR_CH0
  
  // Load the sensitivity value into the transferBytes array.
  for(int i=1; i<8; i++)
  {
    transferBytes[i] = sensitivity;
  }
  // Write the sensitivity values to the register addresses.
  // Since Touch Threshold addresses are consecutive, all 6 bytes can be loaded with one transfer.
  writeRandomBytes(THRESHOLDS_CH0, 8, transferBytes, stopOrRestart);
}

/**
  * @name  setTouchSensitivity
  * @brief  A method which sets the touch threshold on a specified channel, thereby adjusting the sensitivity of the channel.
  *         The Prox threshold for channel 0 is not altered at all.
  * @param  sensitivity   -> A value from 0 to 255 which specifies the touch threshold for the channels, a smaller value means more sensitive.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the writeRandomBytes methods.
  */
void IQS266::setTouchSensitivity(CHANNEL channel, uint8_t sensitivity, bool stopOrRestart)
{
  uint8_t transferBytes[2]; // Temporary array which will hold the bytes to be transferred.
  uint8_t address = 0;
  
  // Select the address which must be read and written.
   switch(channel)
  {
	case CH_0:
		address = THRESHOLDS_CH0;
		break;
	case CH_1: case CH_2:
		address = TOUCH_THR_CH1_CH2;
		break;
	case CH_3: case CH_4:
		address = TOUCH_THR_CH3_CH4;
		break;
	case CH_5: case CH_6:
		address = TOUCH_THR_CH5_CH6;
		break;
  }
  
  // Read the channel 2 bytes at the required address
  readRandomBytes(address, 2, transferBytes, RESTART);    // transferBytes[0] now holds the PROX_THR_CH0
  
  // Load the sensitivity value into the required location in transferBytes array.
  switch(channel)
  {
	case CH_0: case CH_2: case CH_4: case CH_6:
		transferBytes[1] = sensitivity;
		break;
	case CH_1: case CH_3: case CH_5:
		transferBytes[0] = sensitivity;
		break;
  }
  // Write the sensitivity values to the register address.
  // Only one byte was altered, the other remains unchanged.
  writeRandomBytes(address, 2, transferBytes, stopOrRestart);
}

/**
  * @name   setProxSensitivity
  * @brief  A method which sets the proximity threshold, thereby adjusting the sensitivity of the proximity channel which is channel 0.
  * @param  sensitivity   -> A value from 0 to 255 which specifies the proximity threshold for channel 0, a smaller value means more sensitive.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private writeRandomBytes function.
  */
void IQS266::setProxSensitivity(uint8_t sensitivity, bool stopOrRestart)
{
  uint8_t transferBytes[1]; // Temporary array which will hold the bytes to be transferred
  
  transferBytes[0] = sensitivity;
  
  writeRandomBytes(THRESHOLDS_CH0, 1, transferBytes, stopOrRestart);
}

/**
  * @name   enableProxEvent
  * @brief  A function to enable the proximity event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS266::enableProxEvent(bool stopOrRestart)
{
  setEventMaskBits(PROX_EVENT_BIT, stopOrRestart);
}

/**
  * @name   enableTouchEvent
  * @brief  A function to enable the touch event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS266::enableTouchEvent(bool stopOrRestart)
{
  setEventMaskBits(TOUCH_EVENT_BIT, stopOrRestart);
}

/**
  * @name   enableTrackPadEvent
  * @brief  A function to enable the track-pad event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS266::enableTrackPadEvent(bool stopOrRestart)
{
  setEventMaskBits(TP_EVENT_BIT, stopOrRestart);
}

/**
  * @name   enableTapEvent
  * @brief  A function to enable the tap event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS266::enableTapEvent(bool stopOrRestart)
{
  setEventMaskBits(TAP_EVENT_BIT, stopOrRestart);
}

/**
  * @name   enableSwipeEvent
  * @brief  A method which enables the swipe event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS266::enableSwipeEvent(bool stopOrRestart)
{
  setEventMaskBits(SWIPE_EVENT_BIT, stopOrRestart);
}

/**
  * @name   enableAllEvents
  * @brief  A method which enables all the events.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the setEventsMaskBits private function.
  */
void IQS266::enableAllEvents(bool stopOrRestart)
{
  setEventMaskBits(0xFF, stopOrRestart);
}

/**
  * @name   disableProxEvent
  * @brief  A function to disable the proximity event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventMaskBits private function.
  */
void IQS266::disableProxEvent(bool stopOrRestart)
{
  clearEventMaskBits(PROX_EVENT_BIT, stopOrRestart);
}

/**
  * @name   disableTouchEvent
  * @brief  A function to disable the touch event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventMaskBits private function.
  */
void IQS266::disableTouchEvent(bool stopOrRestart)
{
  clearEventMaskBits(TOUCH_EVENT_BIT, stopOrRestart);
}

/**
  * @name   disableTrackPadEvent
  * @brief  A function to disable the track-pad event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventMaskBits private function.
  */
void IQS266::disableTrackPadEvent(bool stopOrRestart)
{
  clearEventMaskBits(TP_EVENT_BIT, stopOrRestart);
}

/**
  * @name   disableTapEvent
  * @brief  A function to disable the tap event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventMaskBits private function.
  */
void IQS266::disableTapEvent(bool stopOrRestart)
{
  clearEventMaskBits(TAP_EVENT_BIT, stopOrRestart);
}

/**
  * @name   disableSwipeEvent
  * @brief  A function to disable the swipe event.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventMaskBits private function.
  */
void IQS266::disableSwipeEvent(bool stopOrRestart)
{
  clearEventMaskBits(SWIPE_EVENT_BIT, stopOrRestart);
}

/**
  * @name   disableAllEvents
  * @brief  A method which disables all the events.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the clearEventsMaskBits private function.
  */
void IQS266::disableAllEvents(bool stopOrRestart)
{
  clearEventMaskBits(0xFF, stopOrRestart);
}

/**
  * @name   enableChannel
  * @brief  A method which enables a channel specified by the user. All current settings are preservec excepts for the channel specified.
  * @param  channel       -> Use the CHANNEL enumerator to specify a channel to be enabled. Passing CH_LP does nothing.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS266::enableChannel(CHANNEL channel, bool stopOrRestart)
{
   uint8_t transferBytes[2]; // Temporary storage for the bytes which are read and written.
  
   // Read both bytes at the CHANNEL_SETTINGS address, the second byte will be altered.
   readRandomBytes(CHANNEL_SETTINGS, 2, transferBytes, RESTART);

   // Set the bit in the second byte for the channel which must be enabled.
   transferBytes[1] |= (1<<channel);
   
   // Write the new value to the ATCTIVE_CHANNELS address
   writeRandomBytes(CHANNEL_SETTINGS, 2, transferBytes, stopOrRestart);
}

/**
  * @name   disableChannel
  * @brief  A method which disables a channel specified by the user. All current settings are preservec excepts for the channel specified.
  * @param  channel       -> Use the CHANNEL enumerator to specify a channel to be enabled. Passing CH_LP does nothing.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS266::disableChannel(CHANNEL channel, bool stopOrRestart)
{
   uint8_t transferBytes[2]; // Temporary storage for the bytes which are read and written.
  
   // Read both bytes at the CHANNEL_SETTINGS address, the second byte will be altered.
   readRandomBytes(CHANNEL_SETTINGS, 2, transferBytes, RESTART);

   // Clear the bit in the second byte for the channel which must be disabled.
   transferBytes[1] &= ~(1<<channel);
   
   // Write the new value to the ATCTIVE_CHANNELS address
   writeRandomBytes(CHANNEL_SETTINGS, 2, transferBytes, stopOrRestart);
}

/**
  * @name   readChannelCounts
  * @brief  A method which reads and returns the counts value of a specific channel
  * @param  channel       -> The enumerator which specifes which channel's counts to return.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns a 16 bit unsigned integer containing the counts value of the channel.
  * @notes  Uses the private readRandomBytes method.
  */
uint16_t IQS266::readChannelCounts(CHANNEL channel, bool stopOrRestart)
{
  uint8_t transferBytes[2];      // A temporary array which will hold all the channel counts low and high bytes.
  uint8_t readAddress = ACF_CH0; // The address of the first channel's counts bytes, this address is adjusted according to the enum passed.
  uint8_t countsLow = 0;         // Temporary storage for the counts low byte.
  uint8_t countsHigh = 0;        // Temporary storage for the counts high byte.
  uint16_t countsReturn = 0;     // The 16bit return value.

  // Adjust the address according to the channel enum which was passed.
  readAddress += channel;
  // Read the bytes with the readRandomBytes method
  readRandomBytes(readAddress, 2, transferBytes, stopOrRestart);
  // Construct the 16bit return value.
  countsLow = transferBytes[0];
  countsHigh = transferBytes[1];
  countsReturn = (uint16_t)(countsLow);
  countsReturn |= (uint16_t)(countsHigh<<8);
  // Return the counts value.
  return countsReturn;
}

/**
  * @name   readChannelLta
  * @brief  A method which reads and returns the LTA value of a specific channel
  * @param  channel       -> The enumerator which specifes which channel's LTA value to return.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns a 16 bit unsigned integer containing the LTA value of the channel.
  * @notes  Uses the private readRandomBytes method.
  */
uint16_t IQS266::readChannelLta(CHANNEL channel, bool stopOrRestart)
{
  uint8_t transferBytes[2];       // A temporary array which will hold all the channel LTA low and high bytes.
  uint8_t readAddress = LTA_CH0;  // The address of the first channel's LTA bytes, this address is adjusted according to the enum passed.
  uint8_t ltaLow = 0;             // Temporary storage for the LTA low byte.
  uint8_t ltaHigh = 0;            // Temporary storage for the LTA high byte.
  uint16_t ltaReturn = 0;         // The 16bit return value.

  // Adjust the address according to the channel enum which was passed.
  readAddress += channel;
  // Read the bytes with the readRandomBytes method
  readRandomBytes(readAddress, 2, transferBytes, stopOrRestart);
  // Construct the 16bit return value.
  ltaLow = transferBytes[0];
  ltaHigh = transferBytes[1];
  ltaReturn = (uint16_t)(ltaLow);
  ltaReturn |= (uint16_t)(ltaHigh<<8);
  // Return the LTA value.
  return ltaReturn;
}

/**
  * @name   readChannelDelta
  * @brief  A method which reads and returns the Delta value of a specific channel
  * @param  channel       -> The enumerator which specifes which channel's Delta value to return.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval Returns a 16 bit unsigned integer containing the Delta value of the channel.
  * @notes  Uses the private readRandomBytes method.
  */
uint16_t IQS266::readChannelDelta(CHANNEL channel, bool stopOrRestart)
{
  uint8_t transferBytes[2];         // A temporary array which will hold all the channel Delta low and high bytes.
  uint8_t readAddress = DELTA_CH0;  // The address of the first channel's Delta bytes, this address is adjusted according to the enum passed.
  uint8_t ltaLow = 0;               // Temporary storage for the Delta low byte.
  uint8_t ltaHigh = 0;              // Temporary storage for the Delta high byte.
  uint16_t ltaReturn = 0;           // The 16bit return value.

  // Adjust the address according to the channel enum which was passed.
  readAddress += channel;
  // Read the bytes with the readRandomBytes method
  readRandomBytes(readAddress, 2, transferBytes, stopOrRestart);
  // Construct the 16bit return value.
  ltaLow = transferBytes[0];
  ltaHigh = transferBytes[1];
  ltaReturn = (uint16_t)(ltaLow);
  ltaReturn |= (uint16_t)(ltaHigh<<8);
  // Return the Delta value.
  return ltaReturn;
}

/**
  * @name   setTapTimerLimit
  * @brief  A method which sets the maximum time allowed for a Tap gesture. The gesture must occur in less time than this setting to be valid.
  * @param  timeLimit     -> A value from 0 to 255 which specifies the Tap timer limit in 2 millsecond increments.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private writeRandomBytes method.
  */
void IQS266::setTapTimerLimit(uint8_t timeLimit, bool stopOrRestart)
{
  uint8_t transferBytes[1]; // Temporary storage for the byte to be transfered.
  // Adjust the time value to a value between 0 and 255.
  if(timeLimit >= 255)
    timeLimit = 255;
  else if (timeLimit <=0)
    timeLimit = 0;

  // Store the time limit to the byte which will be transferred.
  transferBytes[0] = timeLimit;
  // Write the time limit to the correct address.
  writeRandomBytes(TAP_SETTINGS, 1, transferBytes, stopOrRestart);
}

/**
  * @name   setTapThreshold
  * @brief  A method which sets the maximum coordinate length of a square area on the trackpad for Tap gesture. Exceeding this length will not recognise a Tap gesture.
  * @param  timeLimit     -> A value from 0 to 255 which specifies the coordinate length of the maximum square area.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private writeRandomBytes method.
  */
void IQS266::setTapThreshold(uint8_t threshold, bool stopOrRestart)
{
  uint8_t transferBytes[2]; // Temporary storage for the bytes to be transfered.
  // Adjust the time value to a value between 0 and 255.
  if(threshold >= 255)
    threshold = 255;
  else if (threshold <=0)
    threshold = 0;

  // First read the Tap Timer Limit in order not to overwrite it, this will be stored in transferBytes[0].
  readRandomBytes(TAP_SETTINGS, 1, transferBytes, RESTART);
  // Store the the threshold to the second byte which will be transferred.
  transferBytes[1] = threshold;
  // Write the tap settings bytes to the correct address.
  writeRandomBytes(TAP_SETTINGS, 2, transferBytes, stopOrRestart);
}

/**
  * @name   setSwipeTimerLimit
  * @brief  A method which sets the maximum time allowed for a Swipe gesture. The gesture must occur in less time than this setting to be valid.
  * @param  timeLimit     -> A value from 0 to 510 which specifies the Swipe timer limit in 2 millsecond increments.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private writeRandomBytes method.
  */
void IQS266::setSwipeTimerLimit(uint8_t timeLimit, bool stopOrRestart)
{
  uint8_t transferBytes[1]; // Temporary storage for the byte to be transfered.
  // Adjust the time value to a value between 0 and 255.
  if(timeLimit >= 255)
    timeLimit = 255;
  else if (timeLimit <=0)
    timeLimit = 0;

  // Store the time limit to the byte which will be transferred.
  transferBytes[0] = timeLimit;
  // Write the time limit to the correct address.
  writeRandomBytes(SWIPE_SETTINGS, 1, transferBytes, stopOrRestart);
}

/**
  * @name   setSwipeThreshold
  * @brief  A method which sets the minimum coordinate length on the trackpad for Swipe gesture. Swiping for less than the specified threshold will not recognise a Swipe gesture.
  * @param  timeLimit     -> A value from 0 to 255 which specifies the minimum swipe coordinate length.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private writeRandomBytes method.
  */
void IQS266::setSwipeThreshold(uint8_t threshold, bool stopOrRestart)
{
  uint8_t transferBytes[2]; // Temporary storage for the bytes to be transfered.
  // Adjust the time value to a value between 0 and 255.
  if(threshold >= 255)
    threshold = 255;
  else if (threshold <=0)
    threshold = 0;

  // First read the Swipe Timer Limit in order not to overwrite it, this will be stored in transferBytes[0].
  readRandomBytes(SWIPE_SETTINGS, 1, transferBytes, RESTART);
  // Store the the threshold to the second byte which will be transferred.
  transferBytes[1] = threshold;
  // Write the Swipe settings bytes to the correct address.
  writeRandomBytes(SWIPE_SETTINGS, 2, transferBytes, stopOrRestart);
}

/**
  * @name   enableProxDistribute
  * @brief  Enables proximity detection to be distributed over all channels and not just on Ch0.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes method.
  */
void IQS266::enableProxDistribute(bool stopOrRestart)
{
	uint8_t transferBytes[2]; // Temporary array to hold the bytes which will be transferred.

	// Read the 2 bytes at address PROXSettings_23, we are interested in byte 2.
	readRandomBytes(PROXSETTINGS_23, 2, transferBytes, RESTART);
	// Set bit 2 in byte 1, byte 1 is PROX_SETTINGS_3.
	transferBytes[1] |= 0x04;
	// Write both bytes back to the register address.
	writeRandomBytes(PROXSETTINGS_23, 2, transferBytes, stopOrRestart);
}

/**
  * @name   disableProxDistribute
  * @brief  Disables proximity distribution over all channels, proximity will now only be detected by CH0.
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes method.
  */
void IQS266::disableProxDistribute(bool stopOrRestart)
{
	uint8_t transferBytes[2]; // Temporary array to hold the bytes which will be transferred.

	// Read the 2 bytes at address PROXSettings_23, we are interested in byte 2.
	readRandomBytes(PROXSETTINGS_23, 2, transferBytes, RESTART);
	// Clear bit 2 in byte 1, byte 1 is PROX_SETTINGS_3.
	transferBytes[1] &= ~(0x04);
	// Write both bytes back to the register address.
	writeRandomBytes(PROXSETTINGS_23, 2, transferBytes, stopOrRestart);
}


/**************************************************************************************************************/
/*											ADVANCED PUBLIC METHODS										  */
/**************************************************************************************************************/

/**
  * @name   setEventMask
  * @brief  Method which sets the event mask of the IQS266 device. The event mask determines which events will pull ready LOW when they occur.
  * @param  eventMask     -> The new event mask which must be applied.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The entire event mask is set with the new value.
  */
void IQS266::setEventMask(uint8_t eventMask, bool stopOrRestart)
{
  uint8_t transferBytes[1]; // Temporary byte to hold the new event mask.
  
  // Set the new Event Mask
  transferBytes[0] = eventMask;
  // Write the new bytes to the PROX_SETTINGS address
  writeRandomBytes(EVENT_MASK, 1, transferBytes, stopOrRestart);
}

/**
  * @name	setProxAtiTarget
  * @brief	A method which sets the ATI target for the proximity channel (CH_0). This value is multiplied by 8 for the actual counts target.
  * @param	atiTarget 	  -> An 8 bit value which specifies the prox ATI target in increments of 8. Eg. Value = 80, TATI Target = 80 x 8 = 640.
  *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *							 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS266::setProxAtiTarget(uint8_t atiTarget, bool stopOrRestart)
{
	uint8_t transferBytes[2];	// Temporary array which holds the bytes to be transferred.
	
	// First read the 1st byte at the ATI_TARGETS address in order not to change the target for the touch channels, the prox ATI target is byte 1 at the address.
	readRandomBytes(ATI_TARGETS, 1, transferBytes, RESTART);
	// Set the ATI_TARGET_CH0 register, byte 1, to the required value.
	transferBytes[1] = atiTarget;
	// Now write all the bytes back to the ATI_TARGETS address.
	writeRandomBytes(ATI_TARGETS, 2, transferBytes, stopOrRestart);
}

/**
  * @name	setTouchAtiTarget
  * @brief	A method which sets the ATI target for the touch channels (CH_1 to CH_6). This value is multiplied by 8 for the actual counts target.
  * @param	atiTarget 	  -> An 8 bit value which specifies the touch ATI target in increments of 8. Eg. Value = 80, TATI Target = 80 x 8 = 640.
  *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *							 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS266::setTouchAtiTarget(uint8_t atiTarget, bool stopOrRestart)
{
	// No need to read any information since we are just over-writing a value into the first byte at the address.
	// Write the byte into the ATI_TARGETS address.
	writeRandomBytes(ATI_TARGETS, 1, &atiTarget, stopOrRestart);
}

/**
  * @name	setProxBaseValue
  * @brief	A method which sets the base value for channel 0 which is the prox channel.
  * @param	baseValue 	  -> A value between 0 and 15, both inclusive, which sets the base value in increments of 16 ranging from 74 to 314.
  *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *							 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  * 		The base value setting should be lower than the ATI target value.
  */
void IQS266::setProxBaseValue(uint8_t baseValue, bool stopOrRestart)
{
	uint8_t transferBytes[1];	// Temporary array which holds the bytes to be transferred, use an array to remain consistent with other methods.

	// Map the base value to a value between 0 and 15.
	if (baseValue > 15)
		baseValue = 15;

	// First read the byte in order not to change the base value for the touch channels.
	readRandomBytes(CHANNEL_SETTINGS, 1, transferBytes, RESTART);
	// Set the base value for the prox channel.
	transferBytes[0] &= 0xF0; // Preserve the upper 4 bytes, clear the lower 4 bytes.
	transferBytes[0] |= baseValue;
	// Now write the new value to the register address.
	writeRandomBytes(CHANNEL_SETTINGS, 1, transferBytes, stopOrRestart);
}

/**
  * @name	setTouchBaseValue
  * @brief	A method which sets the base value for channels 1 to 6 which are the touch channels.
  * @param	baseValue 	  -> A value between 0 and 15, both inclusive, which sets the base value in increments of 16 ranging from 74 to 314.
  * 						 See the IQS266 datasheet to make the correct selection.
  *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *							 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  * 		The base value setting should be lower than the ATI target value.
  */
void IQS266::setTouchBaseValue(uint8_t baseValue, bool stopOrRestart)
{
	uint8_t transferBytes[1];	// Temporary array which holds the bytes to be transferred, use an array to remain consistent with other methods.

	// Constrain the base value to a maximum value of 15.
	if (baseValue > 15)
		baseValue = 15;
	// Shift the base value to the upper 4 bytes.
	baseValue = (baseValue<<4);

	// First read the byte in order not to change the base value for the touch channels.
	readRandomBytes(CHANNEL_SETTINGS, 1, transferBytes, RESTART);
	// Set the base value for the prox channel.
	transferBytes[0] &= 0x0F; // Preserve the lower 4 bytes, clear the upper 4 bytes.
	transferBytes[0] |= baseValue;
	// Now write the new value to the register address.
	writeRandomBytes(CHANNEL_SETTINGS, 1, transferBytes, stopOrRestart);
}


/**
  * @name	setReportRateNM
  * @brief	A method which sets the normal mode report rate in milliseconds.
  * @param	reportRate 	  -> A value between 0 and 255 which sets the normal mode report rate in milliseconds.
  *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *							 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS266::setReportRateNM(uint8_t reportRate, bool stopOrRestart)
{
	// No need to read any information since we are just over-writing a value into the first byte at the address.
	// Write the byte into the REPORT_RATES address.
	writeRandomBytes(REPORT_RATES, 1, &reportRate, stopOrRestart);
}

/**
  * @name	setReportRateLP
  * @brief	A method which sets the low power mode report rate in increments of 16 milliseconds. 0 disables LP mode.
  * @param	reportRate 	  -> A value between 0 and 255 which sets the low power report rate in milliseconds.
  *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
  *							 Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
  */
void IQS266::setReportRateLP(uint8_t reportRate, bool stopOrRestart)
{
	uint8_t transferBytes[2];	// Temporary array which holds the bytes to be transferred.

	// First read the 1st byte at the REPORT_RATES address in order not to change the report rate for normal mode.
	readRandomBytes(REPORT_RATES, 1, transferBytes, RESTART);
	// Set the report rate for the low power mode.
	transferBytes[1] = reportRate;
	// Now write all the bytes back to the REPORT_RATES address.
	writeRandomBytes(REPORT_RATES, 2, transferBytes, stopOrRestart);
}

/**
 * @name	setNPSegmentRate
 * @brief	A method which sets the low power mode report rate in increments of 16 milliseconds. 0 disables LP mode.
 * @param	segmentRate	  -> A value between 0 and 7. Normal power segment rate = 2^(segmentRate).
 *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
 *							 Use the STOP and RESTART definitions.
 * @retval None.
 * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
 */
void IQS266::setNPSegmentRate(uint8_t segmentRate, bool stopOrRestart)
{
	uint8_t transferBytes[1];	// Temporary array which holds the bytes to be transferred, use an array to remain consistent with other methods.

	// Constrain the segment rate to a maximum value of 7..
	if (segmentRate > 7)
		segmentRate = 7;

	// First read the PROXSETTINGS2 register in order not to change any settings.
	readRandomBytes(PROXSETTINGS_23, 1, transferBytes, RESTART);
	// First clear the lower 3 bytes of PROXSETTINGS2
	transferBytes[0] &= 0xF8; // Preserve all bytes except lower 3.
	// Set the lower 3 bytes to the segment rate value.
	transferBytes[0] |= segmentRate;
	// Write the new value to the PROXSETTINGS2 register.
	writeRandomBytes(PROXSETTINGS_23, 1, transferBytes, stopOrRestart);
}

/**
 * @name	setZoomTimeout
 * @brief	A method which sets the zoom timeout time. This is the time which the device will stay in normal power mode with no activity.
 * 			When the timeout value is reached the device will enter low power mode.
 * @param	segmentRate	  -> A value between 0 and 255 which specifies the timeout time in 500ms increments.
 *			stopOrRestart -> Specifies whether the communications window must be kept open or closed after this communication.
 *							 Use the STOP and RESTART definitions.
 * @retval None.
 * @notes  Uses the private readRandomBytes and writeRandomBytes functions.
 */
void IQS266::setZoomTimeout(uint8_t timeoutPeriod, bool stopOrRestart)
{
	uint8_t transferBytes[2];	// Temporary array which holds the bytes to be transferred.

	// First read the 1st byte at the EVENT_MASK address in order not to change the event mask.
	readRandomBytes(EVENT_MASK, 1, transferBytes, RESTART);
	// Set the ZOOM_TIMEOUT register, byte 1, to the required value.
	transferBytes[1] = timeoutPeriod;
	// Now write all the bytes back to the EVENT_MASK address.
	writeRandomBytes(EVENT_MASK, 2, transferBytes, stopOrRestart);
}

/**
  * @name   disableHaltTimeout
  * @brief  Function to disable halt timeout. 
  * @param  stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The halt HALT_TIMEOUT register accepts a value between 0 and 255 which specifies the halt timeout time in increments of 500ms.
  *         Both a value of 0 and 255 disable the halt timeout function. use setHaltTimeout to enable halt timeout again.
  */
void IQS266::disableHaltTimeout(bool stopOrRestart)
{
  // A value of 255 disables halt timeout.
  uint8_t haltTime = 255;

  // Now write haltTime value (255) to the HALT_TIMEOUT register which is byte 0 at the TIMEOUT_PERIODS address.
  writeRandomBytes(TIMEOUT_PERIODS, 1, &haltTime, stopOrRestart);
}

/**
  * @name   setHaltTimeout
  * @brief  Function to set the Halt Timeout Time in seconds. This is done by setting 
  * @param  haltTimeout   -> The time in seconds to which the timeout must be set, ranging from 1 to 127, 0 disables halt timeout.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  The halt HALT_TIMEOUT register accepts a value between 0 and 255 which specifies the halt timeout time in increments of 500ms.
  *         Both a value of 0 and 255 disable the halt timeout function. Use disableHaltTime to disable halt timeout.
  */
void IQS266::setHaltTimeout(uint8_t haltTime, bool stopOrRestart)
{
  // Map and constrain the haltTime argument.
  if(haltTime > 127)
    haltTime = 254;
  else if(haltTime < 1)
    haltTime = 1;
  else
    haltTime += haltTime;

  // Now write the new halt timeout time value to the HALT_TIMEOUT register which is byte 0 at the TIMEOUT_PERIODS address.
  writeRandomBytes(TIMEOUT_PERIODS, 1, &haltTime, stopOrRestart);
}


/**************************************************************************************************************/
/*                                              PRIVATE METHODS                                                */
/**************************************************************************************************************/

/**
  * @name   toggleReady
  * @brief  A method which toggles the READY pin of an IQS266 device.
  * @param  None.
  * @retval None.
  * @notes  Change pin to OUTPUT, pull LOW, delay, pull HIGH, change pin back to INPUT.
  */
void IQS266::toggleReady(void)
{
  // Change the pinMode to OUTPUT
  pinMode(_readyPin, OUTPUT);
  // Pull the pin LOW
  digitalWrite(_readyPin, LOW);
  // Wait a small amount of time.
  delay(5);
  // Release the ready pin
  digitalWrite(_readyPin, HIGH);
  // Change the pinMode back to INPUT
  pinMode(_readyPin, INPUT);
}

/**
 * @name    readRandomBytes
 * @brief   A methods which reads a specified number of bytes from a specified address and saves it into a user supplied array.
 *          This method is used by all other methods in this class which read data drom the IQS266 device.
 * @param   memoryAddress -> The memory address at which to start reading bytes from.  See the "iqs266_addresses.h" file.
 *          numBytes      -> The number of bytes that must be read.
 *          bytesArray    -> The array which will store the bytes to be read, this array will be overwritten.
 *          stopOrRestart -> A boolean which specifies whether the communication window should remain open or be closed after transfer.
 *                           False keeps it open, true closes it. Use the STOP and RESTART definitions. 
 * @retval  No value is returend, however, the user supplied array is overwritten.
 * @notes   Uses standard arduino "Wire" library which is for I2C communication.
 *          Take note that C++ cannot return an array, therefore, the array which is passed as an argument is overwritten with the required values.
 *          Pass an array to the method by using only its name, e.g. "bytesArray", without the brackets, this basically passes a pointer to the array.
 */
void IQS266::readRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart)
{
  uint8_t i = 0;  // A simple counter to assist with loading bytes into the user supplied array.
  
  // Select the device with the address of "_deviceAddress" and start communication.
  Wire.beginTransmission(_deviceAddress);
  // Send a bit asking for the "memoryAddress" register.
  Wire.write(memoryAddress);
  // Complete the selection and communication initialization.
  Wire.endTransmission(RESTART);  // Restart transmission for reading that follows.
  // The required device has now been selected and it has been told which register to send information from.

  // Request "numBytes" bytes from the device which has address "_deviceAddress"
  do
  {
    Wire.requestFrom(_deviceAddress, numBytes, stopOrRestart);
  }while(Wire.available() == 0);  // Wait for response, this sometimes takes a few attempts

  // Load the received bytes into the array until there are no more 
  while(Wire.available())
  {
    // Load the received bytes into the user supplied array
    bytesArray[i] = Wire.read();
    i++;
  }
// And Bob's your uncle.
}

/**
  * @name   writeRandomBytes
  * @brief  A mthod which writes a specified number of bytes to a specified address, the bytes to write are supplied by means of an array pointer.
  *         This method is used by the all other methods of this class which write data to the IQS266 device.
  * @param  memoryAddress -> The memory address at which to start writing the bytes to. See the "iqs266_addresses.h" file.
  *         numBytes      -> The number of bytes that must be written.
  *         bytesArray    -> The array which stores the bytes which will be written to the memory location.
  *         stopOrRestart -> A boolean which sepcifies whether the communication window should remain open or be closed of transfer.
  *                          False keeps it open, true closes it. Use the STOP and RESTART definitions.
  * @retval No value is returend, only the IQS device registers are altered.
  * @notes  Uses standard arduino "Wire" library which is for I2C communication.
  *         Take note that a full array cannot be passed to a function in C++.
  *         Pass an array to the function by using only its name, e.g. "bytesArray", without the square brackets, this basically passes a pointer to the array.
  *         The values to be written must be loaded into the array prior to passing it to the function.
  */
void IQS266::writeRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart)
{
  // Select the device with the address of "_deviceAddress" and start communication.
  Wire.beginTransmission(_deviceAddress);
  // Specify the memory address where the IQS266 must start saving the data, as designated by the "memoryAddress" variable.
  Wire.write(memoryAddress);
  // Write the bytes as specified in the array which "arrayAddress" pointer points to.
  for(int i=0; i<numBytes; i++)
  {
    Wire.write(bytesArray[i]);
  }
  // End the transmission, user decides to STOP or RESTART.
  Wire.endTransmission(stopOrRestart);
}

/**
  * @name   setEventMaskBits
  * @brief  Function to enable one or more events by setting the appropriate bits in the event mask of the IQS266 device.
  * @param  bitMask       -> A byte with the bits which should be set high.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  None.
  */
void IQS266::setEventMaskBits(uint8_t bitMask, bool stopOrRestart)
{
  uint8_t transferBytes[1]; // Array which will hold the bytes read and written.
  
  // First read the previous settings into the array, keep comms open.
  readRandomBytes(EVENT_MASK, 1, transferBytes, RESTART);
  // set the required bits in the event mask.
  transferBytes[0] |= bitMask;
  // Write the new byte to the EVENT_MASK address.
  writeRandomBytes(EVENT_MASK, 1, transferBytes, stopOrRestart);
}

/**
  * @name   clearEventMaskBits
  * @brief  Function to disable one or more events by clearing the appropriate bits in the event mask of the IQS266 device.
  * @param  bitMask       -> A byte with the bits which should be cleared high.
  *         stopOrRestart -> Specifies whether the communications window must be kept open or closed after retrieving the information.
  *                          Use the STOP and RESTART definitions.
  * @retval None.
  * @notes  None.
  */
void IQS266::clearEventMaskBits(uint8_t bitMask, bool stopOrRestart)
{
  uint8_t transferBytes[1]; // Array which will hold the bytes read and written.
  
  // First read the previous settings into the array, keep comms open.
  readRandomBytes(EVENT_MASK, 1, transferBytes, RESTART);
  // Clear the required bits from the event mask.
  transferBytes[0] &= ~(bitMask);
  // Write the new byte to the EVENT_MASK address.
  writeRandomBytes(EVENT_MASK, 1, transferBytes, stopOrRestart);
}
