/**
  ************************************************************************************
  * @file    IQS266.h
  * @author  Azoteq
  * @version V0.0.0
  * @date    2017-08-23
  * @brief   This file contains the header information for an IQS266 Arduino library.
  *          The goal of the library is to provide easy functionality for 
  *          initializing and using the Azoteq IQS266 capacitive touch device.
  ************************************************************************************
  ************************************************************************************
  * @attention  Makes use of the following standard Arduino libraries:
  *       - Arduino.h   -> included in IQS266.h, comes standard with Arduino
  *       - Wire.h      -> Included in IQS266.h, comes standard with Arduino
  *
  ************************************************************************************
  */

#ifndef IQS266_h
#define IQS266_h

// Include Files
#include "Arduino.h"
#include <Wire.h>
#include "./inc/iqs266_addresses.h"

// Public Global Definitions
#define STOP    true    // For use with Wire.h library. True argument with some functions closes the I2C communication window.
#define RESTART false   // For use with Wire.h library. False argument with some functions keeps the I2C communication window open.
// Event Byte Bits. The EVENTS byte and the EVENT_MASK byte use the same bit positions.
#define PROX_EVENT_BIT      0x01
#define TOUCH_EVENT_BIT     0x02
#define TP_EVENT_BIT        0x04
#define ATI_EVENT_BIT       0x08
#define TAP_EVENT_BIT       0x10
#define SWIPE_EVENT_BIT     0x20
#define LP_EVENT_BIT		0x80
// Gesture Byte Bits
#define TP_BIT              0x01
#define TAP_BIT             0x02
#define SWIPE_UP_BIT        0x04
#define SWIPE_DOWN_BIT      0x08
#define SWIPE_LEFT_BIT      0x10
#define SWIPE_RIGHT_BIT     0x20
// Prox Bits
#define CH0_PROX_BIT		0x01
// Touch Bits
#define CH0_TOUCH_BIT		0x01
#define CH1_TOUCH_BIT		0x02
#define CH2_TOUCH_BIT		0x04
#define CH3_TOUCH_BIT		0x08
#define CH4_TOUCH_BIT		0x10
#define CH5_TOUCH_BIT		0x20
#define CH6_TOUCH_BIT		0x40
// Utility Bits
#define SHOW_RESET_BIT		0x80
#define ACK_RESET_BIT		0x80
#define REDO_ATI_BIT        0x10
#define EVENT_MODE_BIT      0x40

// Type Definitions.
typedef union{
	uint8_t flagByte;
	struct{
		uint8_t lpActive:1;
		uint8_t ignoreGH:1;
		uint8_t inAti:1;
		uint8_t inNPSeg:1;
		uint8_t reserved:1;
		uint8_t atiError:1;
		uint8_t ltaBlocked:1;
		uint8_t shReset:1;
	};

} System_Flags_t;

typedef union{
	uint8_t  flagByte;
	struct {
		uint8_t prox:1;
		uint8_t touch:1;
		uint8_t trackpad:1;
		uint8_t ati:1;
		uint8_t tap:1;
		uint8_t swipe:1;
		uint8_t reserved:1;
		uint8_t lowPower:1;
	};
} Event_Flags_t;

typedef union{
	uint8_t  flagByte;
	struct {
		uint8_t active:1;
		uint8_t tap:1;
		uint8_t swipeUp:1;
		uint8_t swipeDown:1;
		uint8_t swipeLeft:1;
		uint8_t swipeRight:1;
		uint8_t segment0:1;
		uint8_t segment1:1;
	};
} Trackpad_Flags_t;

/*typedef struct{
	uint8_t ch0:1;
	uint8_t ch1:1;
	uint8_t ch2:1;
	uint8_t ch3:1;
	uint8_t ch4:1;
	uint8_t ch5:1;
	uint8_t ch6:1;
} touch_t;

typedef struct {
	uint8_t ch0:1;
} prox_t;*/

typedef union{
	uint8_t flagByte;
	struct{
		uint8_t ch0:1;
		uint8_t ch1:1;
		uint8_t ch2:1;
		uint8_t ch3:1;
		uint8_t ch4:1;
		uint8_t ch5:1;
		uint8_t ch6:1;
		uint8_t reserved:1;
	}touch;
	struct {
		uint8_t reserved:7;
		uint8_t ch0:1;
	} prox;
} Channel_Status_t;

// Public Global Variables
typedef enum {
	CH_0,
	CH_1,
	CH_2,
	CH_3,
	CH_4,
	CH_5,
	CH_6
} CHANNEL;


// Class Prototype
class IQS266{
  public:
    // Public Constructors
    IQS266();

    // Public Variables
    System_Flags_t systemFlags;
    Event_Flags_t events;
    Trackpad_Flags_t trackpadFlags;
    Trackpad_Flags_t gestures;
    Channel_Status_t channels;

    // Public Methods
    bool begin(uint8_t deviceAddressIn, uint8_t readyPinIn);
    bool requestComms(void);
    bool waitForReady(void);
    bool checkReset(bool stopOrRestart);
    void acknowledgeReset(bool stopOrRestart);
    void autoTune(bool stopOrRestart);
    void setStreamMode(bool stopOrRestart);
    void setEventMode(bool stopOrRestart);
    uint8_t readSystemFlags(bool stopOrRestart);
    void updateSystemFlags(bool stopOrRestart);
    uint8_t readEvents (bool stopOrRestart);
    void updateEvents(bool stopOrRestart);
    uint8_t readGestures(bool stopOrRestart);
    void updateGestures(bool stopOrRestart);
    uint8_t readTrackpadFlags(bool stopOrRestart);
    void updateTrackpadFlags(bool stopOrRestart);
	uint8_t readProxByte(bool stopOrRestart);
	uint8_t readTouchByte(bool stopOrRestart);
	uint8_t readChannelStatus(bool stopOrRestart);
    void updateChannelStatus(bool stopOrRestart);
    uint8_t readXCoordinate(bool stopOrRestart);
    uint8_t readYCoordinate(bool stopOrRestart);
    void setTouchSensitivity(uint8_t sensitivity, bool stopOrRestart);
	void setTouchSensitivity(CHANNEL channel, uint8_t sensitivity, bool stopOrRestart);
    void setProxSensitivity (uint8_t sensitivity, bool stopOrRestart);
    void enableProxEvent(bool stopOrRestart);
    void enableTouchEvent(bool stopOrRestart);
    void enableTrackPadEvent(bool stopOrRestart);
    void enableTapEvent(bool stopOrRestart);
    void enableSwipeEvent(bool stopOrRestart);
    void enableAllEvents(bool stopOrRestart);
    void disableProxEvent(bool stopOrRestart);
    void disableTouchEvent(bool stopOrRestart);
    void disableTrackPadEvent(bool stopOrRestart);
    void disableTapEvent(bool stopOrRestart);
    void disableSwipeEvent(bool stopOrRestart);
    void disableAllEvents(bool stopOrRestart);
    void enableChannel(CHANNEL channel, bool stopOrRestart);
    void disableChannel(CHANNEL channel, bool stopOrRestart);
    uint16_t readChannelCounts(CHANNEL channel, bool stopOrRestart);
    uint16_t readChannelLta(CHANNEL channel, bool stopOrRestart);
    uint16_t readChannelDelta(CHANNEL channel, bool stopOrRestart);
    void setTapTimerLimit(uint8_t timeLimit, bool stopOrRestart);
    void setTapThreshold(uint8_t threshold, bool stopOrRestart);
    void setSwipeTimerLimit(uint8_t timeLimit, bool stopOrRestart);
    void setSwipeThreshold(uint8_t threshold, bool stopOrRestart);
    void enableProxDistribute(bool stopOrRestart);
    void disableProxDistribute(bool stopOrRestart);
    
    // Advanced Public Methods
    void setEventMask (uint8_t eventMask, bool stopOrRestart);
	void setProxAtiTarget(uint8_t atiTarget, bool stopOrRestart);
	void setTouchAtiTarget(uint8_t atiTarget, bool stopOrRestart);
	void setProxBaseValue(uint8_t baseValue, bool stopOrRestart);
	void setTouchBaseValue(uint8_t baseValue, bool stopOrRestart);
	void setReportRateNM(uint8_t reportRate, bool stopOrRestart);
	void setReportRateLP(uint8_t reportRate, bool stopOrRestart);
	void setNPSegmentRate(uint8_t segmentRate, bool stopOrRestart);
	void setZoomTimeout(uint8_t timeoutPeriod, bool stopOrRestart);
    void disableHaltTimeout(bool stopOrRestart);
    void setHaltTimeout(uint8_t haltTime, bool stopOrRestart);

 private:
    // Private Variables
    uint8_t _deviceAddress;
    uint8_t _readyPin;

    // Private Methods
    void toggleReady(void);
    void readRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
    void writeRandomBytes(uint8_t memoryAddress, uint8_t numBytes, uint8_t bytesArray[], bool stopOrRestart);
    void setEventMaskBits(uint8_t bitMask, bool stopOrRestart);
    void clearEventMaskBits(uint8_t bitMask, bool stopOrRestart);
};
#endif // IQS266_h  
