/*  IQS266: 6 Channel capacitive touch track-pad example.
 *  This example uses the Azoteq IQS266 capacitive touch controller with a 6 channel trackpad to detect user gestures.
 *  The example starts the IQS266 with its default settings, then sets the event mask and puts the IQS266 into event mode.
 *  No interrupts are used on the ready pin, instead the ready pin is polled.
 *  Setting interrupts on the ready pin can allow the MCU to carry on with other tasks while there are no gestures on the
 *  track-pad.
 *  
 *  Hardware:
 *  - IQS266 with 2x3 track-pad
 *  - Arduino pin 8 - Ready
 *  - Arduino pin A4 - SDA
 *  - Arduino pin A5 - SCL
 */

/***** Library Includes *****/
#include <IQS266.h>

/***** Object Declarations *****/
IQS266 trackPad;

/***** Constant Definitions *****/
#define TRACKPAD_RDY 8
#define TRACKPAD_ADDR 0x44

/***** Global Variables *****/
unsigned char events = 0;
unsigned char gestures = 0;

/***** Setup *****/
void setup() {
  // Set ready pin as INPUT.
  pinMode(TRACKPAD_RDY,INPUT);

  // Start serial communication for output.
  Serial.begin(9600);

  // Initialize the track-pad.
  while(!trackPad.begin(TRACKPAD_ADDR, TRACKPAD_RDY));      // Constructor assigns ready pin an address to IQS266 object, returns 1 if successful.
  while(!trackPad.requestComms());                          // Request communication for additional settings.
  trackPad.setEventMask(((SWIPE_EVENT_BIT)|(TAP_EVENT_BIT)), RESTART);  // Set the event mask to allow only swipe and tap events.
  trackPad.setEventMode(STOP);               // Set the IQS266 into event mode.
  Serial.println("Track-Pad Ready");         // Let the user know initialization is complete.
  
  // Delay to allow for ready to be released.
  delay(10);
}

/***** Main Loop *****/
void loop() {

  // Wait for track-pad ready pin to be pulled low, ready pin is pulled low when an event occurs.
  while(!trackPad.waitForReady());

  // Read the events byte and the gesture byte
  events = trackPad.readEvents(RESTART);
  gestures = trackPad.readGestures(STOP);
  
  if((events & (SWIPE_EVENT_BIT)))      // Check if a swipe event occurred
  {
    if(gestures & SWIPE_UP_BIT)         // Check if it was a upward swipe
      Serial.println("SWIPE UP");
    if(gestures & SWIPE_DOWN_BIT)       // Check if it was a downward swipe
      Serial.println("SWIPE DOWN");
    if(gestures & SWIPE_LEFT_BIT)       // Check if it was a left swipe
      Serial.println("SWIPE LEFT");
    if(gestures & SWIPE_RIGHT_BIT)      // Check if it was a right swipe
      Serial.println("SWIPE RIGHT");
  }
  else if (events & (TAP_EVENT_BIT))    // If there was no swipe event, check if it was a tap event.
  {
    Serial.println("TAP");
  }
}
